﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Reflection;
using System.Diagnostics;
using System.IO;
using System.Windows.Forms;

[assembly: AssemblyTitle("LicxGenerator")]
[assembly: AssemblyProduct("LicxGenerator")]
[assembly: AssemblyCopyright("Copyright © GrapeCity 2015")]

[assembly: AssemblyVersion("1.0.0.0")]
[assembly: AssemblyFileVersion("1.0.0.0")]


namespace LicxGenerator
{
	class Program
	{
        [STAThread]
		static int Main(string[] args)
		{
            StringBuilder errorList = new StringBuilder();
            List<string> licxEntries = new List<string>();
            List<string> processedAssemblyFiles = new List<string>();
            List<Assembly> assemblies = new List<Assembly>();

            // default parameters
            bool recurse = false;
            bool ui = true;
            string path = string.Empty;
            string output = string.Empty;
            string prefix = string.Empty;

            // parse command line arguments
            for (int i = 0; i < args.Length; i++)
            {
                string arg = args[i];
                switch (arg)
                {
                    case "-s":
                        ui = false;
                        break;
                    case "-debug":
                        // attach debugger
                        if (Debugger.IsAttached)
                            Debugger.Break();
                        else
                            Debugger.Launch();
                        break;
                    case "-r":
                        // recurse into sub directories
                        recurse = true;
                        break;
                    case "-p":
                        // fill prefix with next parameter
                        i++;
                        if (i < args.Length)
                        {
                            prefix = args[i];
                        }
                        break;
                    default:
                        if (string.IsNullOrEmpty(path))
                        {
                            path = Path.GetFullPath(arg);
                        }
                        else
                        {
                            output = Path.GetFullPath(arg);
                        }
                        break;
                }
            }

            // validate path
            if (string.IsNullOrEmpty(path))
            {
                if (ui)
                {
                    // show dialog for selecting folder
                    FolderBrowserDialog fbd = new FolderBrowserDialog();
                    fbd.Description = "Select folder with binaries used in your project";
                    fbd.ShowNewFolderButton = false;
                    if (fbd.ShowDialog() == DialogResult.OK)
                    {
                        path = fbd.SelectedPath;
                    }
                }
                if (string.IsNullOrEmpty(path))
                {
                    if (ui)
                    {
                        MessageBox.Show("The source path is empty. No output created.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    }
                    Trace.WriteLine("The source path is empty. No output created.");
                    return 1;
                }
            }

            // validate output file name
            if (string.IsNullOrEmpty(output))
            {
                output = path;
            }
            if (!output.EndsWith("licx"))
            {
                output += "\\licenses.licx";
            }

            // Collect assembly names.
            // To avoid FileNotFoundExeption at examining types from assemblies with not loaded references,
            // preload all assemblies with controls into current app domain.
            if (Directory.Exists(path))
            {
                string[] files = Directory.GetFiles(path, "*.dll", recurse ? SearchOption.AllDirectories : SearchOption.TopDirectoryOnly);
                foreach (string file in files)
                {
                    string filename = Path.GetFileName(file);

                    if (processedAssemblyFiles.Contains(filename, StringComparer.OrdinalIgnoreCase))
                        continue;   // skip duplicates						

                    processedAssemblyFiles.Add(filename);

                    if (!string.IsNullOrEmpty(prefix) && !filename.StartsWith(prefix, StringComparison.OrdinalIgnoreCase))
                    {
                        // skip assemblies with names starting from something other than specified prefix
                        continue;
                    }

                    // pre-load assembly (skip design-time assemblies)
                    if (filename.IndexOf(".design.", StringComparison.OrdinalIgnoreCase) > 0)
                    {
                        // skip design-time assemblies
                        continue;
                    }

                    // pre-load assembly
                    Assembly asm = Assembly.LoadFrom(file);
                    assemblies.Add(asm);
                }
            }
            else // not directory, probably single file
            {
                Assembly asm = Assembly.LoadFrom(path);
                assemblies.Add(asm);
            }

            // process loaded assemblies
            foreach (Assembly asm in assemblies)
            {
                processAssembly(asm, licxEntries, errorList);
            }

            if (licxEntries.Count == 0)
            {
                if (ui)
                {
                    MessageBox.Show("Licensed controls not found. No output created.", "Information", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
                Trace.WriteLine("Licensed controls not found. No output created.");
                return 0;
            }

            // create output file
            if (File.Exists(output))
            {
                File.Delete(output);
            }
            Trace.Listeners.Add(new TextWriterTraceListener(output));

            // fill output
            licxEntries.Sort();

            foreach (string licxEntry in licxEntries)
            {
                Trace.WriteLine(licxEntry);
            }

            if (errorList.Length > 0)
            {
                Trace.WriteLine("");
                Trace.WriteLine(errorList.ToString());
            }

            // end processing
            Trace.Flush();
            return 0;
        }

        static void processAssembly(Assembly asm, List<string> licxEntries, StringBuilder errorList)
        {
            try
            {
                string asmName = asm.FullName.Split(',')[0].Trim();

                Type[] asmTypes;
                try
                {
                    asmTypes = asm.GetTypes();
                }
                catch (ReflectionTypeLoadException ex)
                {
                    // for cases when some referenced assembly is not loaded
                    asmTypes = ex.Types;
                }

                // search for LicenseProviderAttribute
                foreach (Type type in asmTypes)
                {
                    if (type == null)
                    {
                        continue;
                    }
                    object[] attributes = type.GetCustomAttributes(typeof(System.ComponentModel.LicenseProviderAttribute), true);
                    foreach (Attribute att in attributes)
                    {
                        string licxEntry = type.FullName + ", " + asmName;
                        if (!licxEntries.Contains(licxEntry, StringComparer.OrdinalIgnoreCase))
                            licxEntries.Add(licxEntry);
                    }
                }
            }
            catch (Exception e)
            {
                errorList.AppendLine();
                errorList.AppendLine("Unable to process assembly: " + asm.FullName);
                errorList.AppendLine("  " + e.Message);
            }
        }
	}
}

